@@ -7,7 +7,7 @@ from rich.console import Console
 from rich.syntax import Syntax
 
 from piperider_cli import __version__, sentry_dns, sentry_env, event
-from piperider_cli.adapter import DbtAdapter
+import piperider_cli.dbtutil as dbtutil
 from piperider_cli.assertion_generator import AssertionGenerator
 from piperider_cli.cloud_connector import CloudConnector
 from piperider_cli.compare_report import CompareReport
@@ -114,7 +114,7 @@ def init(**kwargs):
         if dbt_project_dir:
             dbt_project_path = os.path.join(dbt_project_dir, "dbt_project.yml")
         else:
-            dbt_project_path = DbtAdapter.search_dbt_project_path()
+            dbt_project_path = dbtutil.search_dbt_project_path()
 
     if dbt_project_path:
         console.print(f'[[bold green] DBT [/bold green]] Use the existing dbt project file: {dbt_project_path}')
@@ -152,11 +152,8 @@ def diagnose(**kwargs):
 @click.option('--datasource', default=None, type=click.STRING, help='Datasource to use.', metavar='DATASOURCE_NAME')
 @click.option('--table', default=None, type=click.STRING, help='Table to use.', metavar='TABLE_NAME')
 @click.option('--output', '-o', default=None, type=click.STRING, help='Directory to save the results.')
-@click.option('--no-interaction', is_flag=True, help='Disable interactive mode.')
 @click.option('--skip-report', is_flag=True, help='Skip generating report.')
-@click.option('--skip-recommend', is_flag=True, help='Skip recommending assertions.')
-@click.option('--dbt-test', is_flag=True, help='Run dbt test.')
-@click.option('--dbt-build', is_flag=True, help='Run dbt build.')
+@click.option('--dbt-state', default=None, help='Directory of the the dbt state.')
 @click.option('--report-dir', default=None, type=click.STRING, help='Use a different report directory.')
 @click.option('--upload', is_flag=True, help='Upload the report to the PipeRider Cloud.')
 @add_options(debug_option)
@@ -167,17 +164,12 @@ def run(**kwargs):
     table = kwargs.get('table')
     output = kwargs.get('output')
     skip_report = kwargs.get('skip_report')
-    skip_recommend = kwargs.get('skip_recommend')
-    run_dbt_test = kwargs.get('dbt_test')
-    run_dbt_build = kwargs.get('dbt_build')
-    dbt_command = 'build' if run_dbt_build else 'test' if run_dbt_test else ''
+    dbt_state_dir = kwargs.get('dbt_state')
     ret = Runner.exec(datasource=datasource,
                       table=table,
                       output=output,
-                      interaction=not kwargs.get('no_interaction'),
                       skip_report=skip_report,
-                      skip_recommend=skip_recommend,
-                      dbt_command=dbt_command,
+                      dbt_state_dir=dbt_state_dir,
                       report_dir=kwargs.get('report_dir'))
 
     if ret in (0, EC_ERR_TEST_FAILED):
@@ -194,12 +186,20 @@ def run(**kwargs):
 
 @cli.command(short_help='Generate recommended assertions.', cls=TrackCommand)
 @click.option('--input', default=None, type=click.Path(exists=True), help='Specify the raw result file.')
+@click.option('--no-recommend', is_flag=True, help='Generate assertions templates only.')
 @click.option('--report-dir', default=None, type=click.STRING, help='Use a different report directory.')
+@click.option('--table', default=None, type=click.STRING, help='Generate assertions for the given table')
 @add_options(debug_option)
 def generate_assertions(**kwargs):
     'Generate recommended assertions based on the latest result. By default, the profiling result will be loaded from ".piperider/outputs".'
-    input = kwargs.get('input')
-    AssertionGenerator.exec(input=input, report_dir=kwargs.get('report_dir'))
+    input_path = kwargs.get('input')
+    report_dir = kwargs.get('report_dir')
+    no_recommend = kwargs.get('no_recommend')
+    table = kwargs.get('table')
+    ret = AssertionGenerator.exec(input_path=input_path, report_dir=report_dir, no_recommend=no_recommend, table=table)
+    if ret != 0:
+        sys.exit(ret)
+    return ret
 
 
 @cli.command(short_help='Generate a report.', cls=TrackCommand)
@@ -220,6 +220,9 @@ def generate_report(**kwargs):
               help='Specify the datasource.')
 @click.option('--output', '-o', default=None, type=click.STRING, help='Directory to save the results.')
 @click.option('--report-dir', default=None, type=click.STRING, help='Use a different report directory.')
+@click.option('--tables-from', default='all',
+              type=click.Choice(['all', 'target-only', 'base-only'], case_sensitive=False),
+              help='Show table comparison from base or target.')
 @add_options(debug_option)
 def compare_reports(**kwargs):
     'Compare two existing reports selected in interactive mode or by option.'
@@ -228,8 +231,9 @@ def compare_reports(**kwargs):
     b = kwargs.get('target')
     last = kwargs.get('last')
     datasource = kwargs.get('datasource')
+    tables_from = kwargs.get('tables_from')
     CompareReport.exec(a=a, b=b, last=last, datasource=datasource,
-                       report_dir=kwargs.get('report_dir'), output=kwargs.get('output'),
+                       report_dir=kwargs.get('report_dir'), output=kwargs.get('output'), tables_from=tables_from,
                        debug=kwargs.get('debug', False))
 
 
