@@ -16,17 +16,16 @@ from rich.table import Table
 from sqlalchemy import inspect
 from sqlalchemy.exc import NoSuchTableError
 
+import piperider_cli.dbtutil as dbtutil
 from piperider_cli import convert_to_tzlocal, datetime_to_str, clone_directory, \
     raise_exception_when_directory_not_writable
 from piperider_cli import event
-import piperider_cli.dbtutil as dbtutil
 from piperider_cli.assertion_engine import AssertionEngine
 from piperider_cli.assertion_engine.recommender import RECOMMENDED_ASSERTION_TAG
 from piperider_cli.configuration import Configuration
-from piperider_cli.datasource import DataSource
 from piperider_cli.exitcode import EC_ERR_TEST_FAILED
 from piperider_cli.filesystem import FileSystem
-from piperider_cli.profiler import Profiler, ProfilerEventHandler
+from piperider_cli.profiler import Profiler, ProfilerEventHandler, ProfileSubject
 
 
 class RunEventPayload:
@@ -263,7 +262,7 @@ def _show_dbt_test_result_summary(dbt_test_results, table: str = None):
             test_count[t] = dict(total=0, failed=0)
 
         test_count[t]['total'] += 1
-        test_count[t]['failed'] += 1 if r.get('status') != 'passed' else 0
+        test_count[t]['failed'] += 1 if r.get('status') == 'failed' else 0
 
     for t in test_count:
         ascii_table.add_row(
@@ -605,32 +604,28 @@ class Runner():
 
         tables = None
         dbt_test_results = None
-        dbt_test_results_compatible = None
         if dbt_state_dir:
             if not dbtutil.is_dbt_state_ready(dbt_state_dir):
                 console.print(
                     f"[bold red]Error:[/bold red] No available 'manifest.json' or 'run_results.json' under '{dbt_state_dir}'")
                 return 1
 
-            includes = dbtutil.get_dbt_state_candidate(dbt_state_dir, default_schema)
-
-            if configuration.includes is not None:
-                # intersect 'includes' and 'configuration.includes'
-                includes = [include for include in includes if include in configuration.includes]
-
-            configuration.includes = includes
-
-            dbt_test_results, dbt_test_results_compatible = dbtutil.get_dbt_state_tests_result(dbt_state_dir,
-                                                                                               default_schema)
+            tables = dbtutil.get_dbt_state_candidate(dbt_state_dir)
+            dbt_test_results = dbtutil.get_dbt_state_tests_result(dbt_state_dir)
 
         if table:
-            tables = [table]
+            if len(table.split('.')) == 2:
+                schema, table_name = table.split('.')
+                tables = [ProfileSubject(table_name, schema, table_name)]
+            else:
+                tables = [ProfileSubject(table, default_schema, table)]
             # cli --table is specified, no inclusion and exclusion applied
             configuration.includes = None
             configuration.excludes = None
 
         run_result = {}
-        profiler = Profiler(engine, RichProfilerEventHandler(tables if tables else available_tables), configuration)
+        available_tables = [t.table for t in tables] if tables else available_tables
+        profiler = Profiler(engine, RichProfilerEventHandler(available_tables), configuration)
         try:
             profiler_result = profiler.profile(tables)
             run_result.update(profiler_result)
@@ -659,14 +654,7 @@ class Runner():
 
         console.rule('Summary')
 
-        if dbt_test_results_compatible:
-            for k, v in dbt_test_results_compatible.items():
-                if k not in run_result['tables']:
-                    continue
-                run_result['tables'][k]['dbt_assertion_result'] = v
-
         for t in run_result['tables']:
-            run_result['tables'][t]['piperider_assertion_result'] = _transform_assertion_result(t, assertion_results)
             _clean_up_profile_null_properties(run_result['tables'][t])
         _show_summary(run_result, assertion_results, assertion_exceptions, dbt_test_results)
         _show_recommended_assertion_notice_message(console, assertion_results)
@@ -676,7 +664,7 @@ class Runner():
                 console.print(
                     f"[bold red]Error:[/bold red] No available 'manifest.json' or 'run_results.json' under '{dbt_state_dir}'")
                 return 1
-            dbtutil.append_descriptions(run_result, dbt_state_dir, default_schema)
+            dbtutil.append_descriptions(run_result, dbt_state_dir)
         _append_descriptions_from_assertion(run_result)
 
         run_result['id'] = run_id
